#ifndef __ROCCAT_MACRO_EDITOR_ADVANCED_LIST_STORE_H__
#define __ROCCAT_MACRO_EDITOR_ADVANCED_LIST_STORE_H__

/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "roccat/roccat.h"
#include <gtk/gtk.h>

G_BEGIN_DECLS

#define ROCCAT_MACRO_EDITOR_ADVANCED_LIST_STORE_TYPE (roccat_macro_editor_advanced_list_store_get_type())
#define ROCCAT_MACRO_EDITOR_ADVANCED_LIST_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), ROCCAT_MACRO_EDITOR_ADVANCED_LIST_STORE_TYPE, RoccatMacroEditorAdvancedListStore))
#define IS_ROCCAT_MACRO_EDITOR_ADVANCED_LIST_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), ROCCAT_MACRO_EDITOR_ADVANCED_LIST_STORE_TYPE))

typedef struct _RoccatMacroEditorAdvancedListStore RoccatMacroEditorAdvancedListStore;
typedef struct _RoccatMacroEditorAdvancedListStorePrivate RoccatMacroEditorAdvancedListStorePrivate;

struct _RoccatMacroEditorAdvancedListStore {
	GtkListStore parent;
	RoccatMacroEditorAdvancedListStorePrivate *priv;
};

GType roccat_macro_editor_advanced_list_store_get_type(void);

RoccatMacroEditorAdvancedListStore *roccat_macro_editor_advanced_list_store_new(void);

void roccat_macro_editor_advanced_list_store_set_modified(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store, gboolean state);
gboolean roccat_macro_editor_advanced_list_store_get_modified(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store);

GtkWidget *roccat_macro_editor_advanced_list_store_get_label_widget(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store, GtkTreeIter *iter);
GtkWidget *roccat_macro_editor_advanced_list_store_get_lower_widget(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store, GtkTreeIter *iter);
GtkWidget *roccat_macro_editor_advanced_list_store_get_upper_widget(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store, GtkTreeIter *iter);
GtkWidget *roccat_macro_editor_advanced_list_store_get_scale_widget(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store, GtkTreeIter *iter);
gdouble roccat_macro_editor_advanced_list_store_get_all_max(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store);
glong roccat_macro_editor_advanced_list_store_get_abs_time(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store);
void roccat_macro_editor_advanced_list_store_set_all_max(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store, gdouble value);

RoccatMacroKeystrokes *roccat_macro_editor_advanced_list_store_get_keystrokes(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store);
void roccat_macro_editor_advanced_list_store_set_keystrokes(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store, RoccatMacroKeystrokes const *macro_keystrokes);
void roccat_macro_editor_advanced_list_store_add_keystroke(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store, guint key, guint action, glong abs_time);

void roccat_macro_editor_advanced_list_store_clear(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store);
gboolean roccat_macro_editor_advanced_list_store_empty(RoccatMacroEditorAdvancedListStore *macro_editor_advanced_list_store);

G_END_DECLS

#endif
